@@ -11,7 +11,6 @@ from Bio.Blast import NCBIXML
 from Bio import SeqIO
 
 import logging
-from funid.src.visualize import plot_pca, plot_heatmap
 from funid.src.ext import mmseqs
 
 
@@ -38,19 +37,10 @@ def get_naive_group(V):
     )
 
 
-def append_concatenated_query_group(V):
-
-    group_dict = {}
-    for funinfo in V.list_FI:
-        group_dict[funinfo.hash] = funinfo.adjusted_group
-
-    V.cSR["query_group"] = V.cSR["qseqid"].apply(lambda x: group_dict.get(x))
-
-    return V
-
-
+# Append query_group information column to dict_gene_SR
 def append_query_group(V):
 
+    # For normal gene matrix
     group_dict = {}
     for FI in V.list_FI:
         group_dict[FI.hash] = FI.adjusted_group
@@ -60,6 +50,13 @@ def append_query_group(V):
             lambda x: group_dict.get(x)
         )
 
+    # For concatenated gene matrix
+    group_dict = {}
+    for funinfo in V.list_FI:
+        group_dict[funinfo.hash] = funinfo.adjusted_group
+
+    V.cSR["query_group"] = V.cSR["qseqid"].apply(lambda x: group_dict.get(x))
+
     return V
 
 
@@ -107,10 +104,13 @@ def assign_gene(result_dict, V, cutoff=0.99):
             # if only 1 gene available, take it
             if gene_count == 1:
                 funinfo.update_seq(gene_list[0], seq)
+                logging.info(
+                    f" Query seq in {funinfo.id} has assigned to {gene_list[0]}."
+                )
             # if no gene matched, warn it
             elif gene_count == 0:
                 logging.warning(
-                    f" Query seq in {funinfo.id} cannot be assigned to gene. Check sequence. Skipping {funinfo.id}"
+                    f" Query seq in {funinfo.id} cannot be assigned to any gene. Check sequence. Skipping {funinfo.id}"
                 )
             elif gene_count >= 2:
                 logging.warning(
@@ -134,6 +134,7 @@ def cluster(FI, df_search, V, path, opt):
 
     # If no evidence available, return it
     if df_search is None:
+        logging.warning(f"No adjusted_group assigned to {FI}")
         FI.adjusted_group = FI.group
         return FI, None
     # for db sequence with group, retain it
@@ -192,16 +193,11 @@ def cluster(FI, df_search, V, path, opt):
 # Append outgroup to given group-gene dataset by search matrix
 def append_outgroup(V, df_search, gene, group, path, opt):
 
-    logging.info(f"Appending outgroup on group:{group}, Gene:{gene}")
+    logging.info(f"Appending outgroup on group: {group}, Gene: {gene}")
 
-    # funinfo for designated groups
-    list_FI_return = [
-        funinfo for funinfo in V.list_FI if funinfo.adjusted_group == group
-    ]
-
-    # list_FI = copy.deepcopy(V.list_FI)
-    list_FI = V.list_FI
+    list_FI = copy.deepcopy(V.list_FI)
 
+    # In multiprocessing, delete V for memory
     del V
 
     # ready for by sseqid hash, which group to append
@@ -276,7 +272,7 @@ def append_outgroup(V, df_search, gene, group, path, opt):
                     funinfo_dict[cutoff_df["sseqid"][n]]
                 )
 
-        # if enough outgroup sequences found
+        # if enough outgroup sequences found while running
         if len(outgroup_dict[subject_group]) >= opt.maxoutgroup:
             text_outgroup_list = "\n ".join(
                 [FI.id for FI in outgroup_dict[subject_group]]
@@ -285,11 +281,17 @@ def append_outgroup(V, df_search, gene, group, path, opt):
                 f"Outgroup [{subject_group}] selected to [{group}]\n {text_outgroup_list}"
             )
 
+            # Get database sequences in ambiguous position
+            ambiguous_group = []
+            for _group in outgroup_dict:
+                if _group != subject_group:
+                    ambiguous_group += outgroup_dict[_group]
+
             return (
-                gene,
                 group,
+                gene,
                 outgroup_dict[subject_group],
-                outgroup_dict[subject_group] + list_FI_return,
+                ambiguous_group,
             )
         else:
             if len(outgroup_dict[subject_group]) > max_cnt:
@@ -305,21 +307,21 @@ def append_outgroup(V, df_search, gene, group, path, opt):
             f"Final outgroup selection for group {group} : {outgroup_dict[max_group]}"
         )
 
-        return (
-            gene,
-            group,
-            outgroup_dict[max_group],
-            outgroup_dict[max_group] + list_FI_return,
-        )
+        # Get database sequences in ambiguous position
+        ambiguous_group = []
+        for _group in outgroup_dict:
+            if _group != subject_group:
+                ambiguous_group += outgroup_dict[_group]
+
+        return (group, gene, outgroup_dict[max_group], ambiguous_group)
     else:
         logging.warning(f"No outgroup sequence available for {group}")
-        return (gene, group, [], list_FI_return)
+        return (group, gene, [], [])
 
 
 def group_cluster_opt_generator(V, opt, path):
 
     # cluster(FO, df_search, V, path, opt)
-
     if len(V.list_qr_gene) == 0:
         logging.error(
             "In group_cluster_option_generator, no possible query genes were selected"
@@ -346,12 +348,18 @@ def group_cluster_opt_generator(V, opt, path):
             )
 
         list_multigene_FI = []
+
         for FI in list_FI:
             # If only one gene
             if len(list(FI.seq.keys())) == 1:
                 gene = list(FI.seq.keys())[0]
-                appropriate_df = df_group_dict[gene].get_group(FI.hash)
-                V.opt_cluster.append((FI, appropriate_df, V, path, opt))
+                try:
+                    appropriate_df = df_group_dict[gene].get_group(FI.hash)
+                    V.opt_cluster.append((FI, appropriate_df, V, path, opt))
+                except:
+                    logging.warning(
+                        f"Cannot assign group to {FI} {gene}, removing from analysis"
+                    )
 
             # If no sequence available
             elif len(list(FI.seq.keys())) == 0:
@@ -397,67 +405,42 @@ def group_cluster_opt_generator(V, opt, path):
 # opts ready for multithreading in outgroup append
 def outgroup_append_opt_generator(V, path, opt):
 
+    opt_append_outgroup = []
+
     # if concatenated analysis is false
     # Assign different outgroup for each dataset
-    for gene in opt.gene:
-        for group in V.list_group:
-            print(f"Checking {group} {gene} dataset")
-            if V.exist_dataset(group, gene) is True:
-                print(f"{group} {gene} dataset exists!")
+    for group in V.dict_dataset:
+        for gene in V.dict_dataset[group]:
+            if gene != "concatenated":
                 try:
                     df = V.dict_gene_SR[gene]
                     df_group = df.groupby(df["query_group"])
                     df_group_ = df_group.get_group(group)
                     # Generating outgroup opt for multiprocessing
-                    V.opt_append_og.append((V, df_group_, gene, group, path, opt))
+                    opt_append_outgroup.append((V, df_group_, gene, group, path, opt))
                 except:
                     logging.warning(
-                        f"{group} / {gene} dataset exists, but cannot append outgroup due to no corresponding search result. Removing from further analysis"
+                        f"{group} / {gene} dataset exists, but cannot append outgroup due to no corresponding search result"
                     )
-                    V.dict_dataset[group].pop(gene)
-                    logging.debug(f"After pop {V.dict_dataset[group]}")
 
     # if concatenated analysis is true
     # concatenated
-    if opt.concatenate is True:
-        for group in V.list_group:
-            if V.exist_dataset(group, "concatenated") is True:
-                try:
-                    df = V.cSR
-                    df_group = df.groupby(df["query_group"])
-                    df_group_ = df_group.get_group(group)
-                    # Generating outgroup opt for multiprocessing
-                    V.opt_append_og.append(
-                        (V, df_group_, "concatenated", group, path, opt)
-                    )
-                except:
-                    logging.warning(
-                        f"{group} / concatenated dataset exists, but cannot append outgroup due to no corresponding search result. Removing from further analysis"
-                    )
-                    V.dict_dataset[group].pop("concatenated")
-                    logging.debug(f"After pop {V.dict_dataset[concatenated]}")
+    for group in V.dict_dataset:
+        if "concatenated" in V.dict_dataset[group]:
+            try:
+                df = V.cSR
+                df_group = df.groupby(df["query_group"])
+                df_group_ = df_group.get_group(group)
+                # Generating outgroup opt for multiprocessing
+                opt_append_outgroup.append(
+                    (V, df_group_, "concatenated", group, path, opt)
+                )
+            except:
+                logging.warning(
+                    f"{group} / concatenated dataset exists, but cannot append outgroup due to no corresponding search result"
+                )
 
-    return V
-
-
-# multiprocessing run result collector for outgrouping
-def outgroup_result_collector(V):
-
-    for result in V.rslt_append_og:
-
-        gene = result[0]
-        group = result[1]
-        result_outgroup_list = result[2]
-        result_list_group = result[3]
-
-        # append outgroup lists
-        V.dict_dataset[group][gene].list_og_FI = result_outgroup_list
-
-        # Give default species info
-        for FI in V.dict_dataset[group][gene].list_og_FI:
-            FI.bygene_species[gene] = FI.ori_species
-
-    return V
+    return opt_append_outgroup
 
 
 ## Main cluster pipe
@@ -467,7 +450,7 @@ def pipe_cluster(V, opt, path):
 
         logging.info("group clustering")
 
-        # cluster 1opt generation for multiprocessing
+        # cluster opt generation for multiprocessing
         V = group_cluster_opt_generator(V, opt, path)
 
         # run multiprocessing start
@@ -479,10 +462,9 @@ def pipe_cluster(V, opt, path):
         else:
             # non-multithreading mode for debugging
             V.rslt_cluster = [cluster(*o) for o in V.opt_cluster]
-
         # gather cluster result
-        for r in V.rslt_cluster:
-            FI = r[0]
+        for cluster_result in V.rslt_cluster:
+            FI = cluster_result[0]
             logging.debug((FI.id, FI.datatype, FI.group, FI.adjusted_group))
 
         # replace group assigning result
@@ -494,15 +476,21 @@ def pipe_cluster(V, opt, path):
         V.list_FI = [
             FI for FI in V.list_FI if not (FI.hash in replace_hash_FI)
         ] + replace_FI
+        # For syncyhronizing FI in dict_hash_FI to prevent error
+        for FI in replace_FI:
+            V.dict_hash_FI[FI.hash] = FI
 
         V.list_group = list(set([r[1] for r in V.rslt_cluster if (not (r[1] is None))]))
 
-        # For db sequences, maintain group
         if opt.queryonly is True:
             for FI in V.list_FI:
                 if FI.datatype == "db":
                     FI.adjusted_group = FI.group
 
+            # Update dict_hash_FI
+            for FI in V.list_FI:
+                V.dict_hash_FI[FI.hash] = FI
+
         for FI in V.list_FI:
             logging.debug((FI.id, FI.datatype, FI.group, FI.adjusted_group))
 
@@ -516,23 +504,38 @@ def pipe_cluster(V, opt, path):
         for FI in V.list_FI:
             FI.adjusted_group = FI.group
 
+        # Update dict_hash_FI
+        for FI in V.list_FI:
+            V.dict_hash_FI[FI.hash] = FI
+
     return V, opt, path
 
 
 ## Main outgroup appending pipeline
 def pipe_append_outgroup(V, path, opt):
 
-    V = outgroup_append_opt_generator(V, path, opt)
+    opt_append_outgroup = outgroup_append_opt_generator(V, path, opt)
 
     # run multiprocessing start
     if opt.verbose < 3:
         p = mp.Pool(opt.thread)
-        V.rslt_append_og = p.starmap(append_outgroup, V.opt_append_og)
+        result_append_outgroup = p.starmap(append_outgroup, opt_append_outgroup)
         p.close()
         p.join()
 
     else:
         # non-multithreading mode for debugging
-        V.rslt_append_og = [append_outgroup(*o) for o in V.opt_append_og]
+        result_append_outgroup = [append_outgroup(*o) for o in opt_append_outgroup]
+
+    # append outgroup by running result
+    # (group, gene, outgroup, ambiguous_group)
+    for result in result_append_outgroup:
+        group = result[0]
+        gene = result[1]
+        outgroup = result[2]
+        ambiguous_group = result[3]
+
+        V.dict_dataset[group][gene].list_og_FI = outgroup
+        V.dict_dataset[group][gene].list_db_FI += ambiguous_group
 
     return V, path, opt
