@@ -4,6 +4,7 @@ from funid.src import tree_interpretation
 from funid.src.tool import initialize_path, get_genus_species, mkdir
 from funid.src.hasher import encode, decode
 from funid.src.reporter import Singlereport
+from funid.src.patch import patch
 import pandas as pd
 import sys, os
 import shutil
@@ -27,6 +28,7 @@ def pipe_module_tree_interpretation(
     hash_dict = V.dict_hash_name
     query_list = V.dict_dataset[group][gene].list_qr_FI
     outgroup = V.dict_dataset[group][gene].list_og_FI
+
     # for unexpectively included sequence during clustering
     db_list = list(
         set([FI for FI in V.list_FI if FI.datatype == "db"])
@@ -66,14 +68,9 @@ def pipe_module_tree_interpretation(
 
     # Main phase
     # calculate zero distance with alignment
-    if gene != "concatenated":
-        tree_info.calculate_zero(
-            f"{path.out_alignment}/{opt.runname}_hash_trimmed_{group}_{gene}.fasta"
-        )
-    else:
-        tree_info.calculate_zero(
-            f"{path.out_alignment}/{opt.runname}_hash_trimmed_{group}_concatenated.fasta"
-        )
+    tree_info.calculate_zero(
+        f"{path.out_alignment}/{opt.runname}_hash_trimmed_{group}_{gene}.fasta"
+    )
 
     # Reroot outgroup and save original tree into image
     tree_info.reroot_outgroup(
@@ -96,7 +93,7 @@ def pipe_module_tree_interpretation(
 
     # Reconstruct flat branches if option given
     if opt.solveflat is True:
-        tree_info.t = tree_info.reconstruct(tree_info.t.copy("newick"), gene)
+        tree_info.t = tree_info.reconstruct(tree_info.t.copy("newick"), gene, opt)
 
     # reorder tree for pretty look
     tree_info.t.ladderize(direction=1)
@@ -110,14 +107,12 @@ def pipe_module_tree_interpretation(
 ### synchronize sp. numbers from multiple dataset
 # to use continuous sp numbers over trees
 # Seperated because this step cannot be done by multiprocessing
-def synchronize(V, tree_info_list):
+def synchronize(V, path, tree_info_list):
 
     # Get all available genus from tree_info
     def get_possible_genus(tree_info):
         return set([taxon[0] for taxon in tree_info.collapse_dict])
 
-    tree_info_dict = {}
-
     # get available groups per genus
     tree_info_dict = {}
 
@@ -132,27 +127,32 @@ def synchronize(V, tree_info_list):
                 tree_info_dict[genus][tree_info.group][tree_info.gene] = tree_info
             else:
                 logging.error("DEVELOPMENTAL ERROR, DUPLICATED TREE_INFO")
-                raise exception
+                raise Exception
 
     for genus in tree_info_dict.keys():
-        # cnt number that should be added
+        # cnt number that should be added (sp. {cnt_sp_adder})
         cnt_sp_adder = 0
+
+        # sp. numbers should be counted by genus
         for group in sorted(list(tree_info_dict[genus].keys())):
             if not ("concatenated" in tree_info_dict[genus][group]):
-                logging.error("DEVELOPMENTAL ERROR, NO CONCATENATED ANALYSIS")
-                raise exception
+                # Now concatenated analysis gets mandatory
+                logging.info(
+                    f"No concatenated dataset for {genus} {group}. Passing synchronizing"
+                )
             else:
-                # Start with concatenated
+                # Start with concatenated to define standard sp numbers
                 tree_info = tree_info_dict[genus][group]["concatenated"]
 
-                # Get all sp taxon list
+                # Get all sp. taxon list
                 taxon_set = set()
                 for taxon in tree_info.collapse_dict.keys():
                     if "sp." in taxon[1]:
                         taxon_set.add(taxon)
-
-                hash_dict = {}
                 # Make one hash - one sp dict pair
+                hash_dict = {}
+
+                # Add sp numbers to synchronize through genus
                 for taxon in taxon_set:
                     for leaf in tree_info.collapse_dict[taxon][0].leaf_list:
                         hash_dict[leaf[0]] = (
@@ -160,7 +160,8 @@ def synchronize(V, tree_info_list):
                             f"sp. {str(int(taxon[1].split(' ')[1]) + cnt_sp_adder)}",
                         )
 
-                ## Change it with cnt_sp_adder
+                ## Update tree_info.collapse_dict with cnt_sp_adder
+                # Perform in two steps in order to take collapse_dict safe
                 for taxon in taxon_set:
                     tree_info.collapse_dict[
                         (
@@ -169,7 +170,6 @@ def synchronize(V, tree_info_list):
                         )
                     ] = tree_info.collapse_dict.pop(taxon)
 
-                # Performing in two steps in order to tkae collapse_dict safe
                 for taxon in taxon_set:
                     tree_info.collapse_dict[
                         (
@@ -183,45 +183,130 @@ def synchronize(V, tree_info_list):
                         )
                     )
 
+                    for collapse_info in tree_info.collapse_dict[
+                        (
+                            taxon[0],
+                            f"sp. {str(int(taxon[1].split(' ')[1]) + cnt_sp_adder)}",
+                        )
+                    ]:
+                        collapse_info.taxon = (
+                            taxon[0],
+                            f"sp. {str(int(taxon[1].split(' ')[1]) + cnt_sp_adder)}",
+                        )
+
                 # Now solve other genes
                 for gene in tree_info_dict[genus][group]:
                     if gene != "concatenated":
+
+                        # {original_taxon : taxon_to_be_changed}
                         bygene_taxon_dict = {}
 
+                        # Grab the gene tree
                         tree_info = tree_info_dict[genus][group][gene]
 
                         # Synchronize bygene taxon name to concatenated
                         for taxon in tree_info.collapse_dict:
+                            bygene_taxon_dict[taxon] = set()
 
-                            # Do not change non- sp.
-                            if "sp." in taxon[1]:
+                            if not ("sp.") in taxon:
+                                bygene_taxon_dict[taxon].add(taxon[1].strip())
 
-                                hash_list = [
-                                    leaf[0]
-                                    for leaf in tree_info.collapse_dict[taxon][
-                                        0
-                                    ].leaf_list
+                            # for sp. clade
+                            # if "sp." in taxon[1]:
+                            # Get all hash of designated taxon leaf
+                            clade = tree_info.collapse_dict[taxon][0]
+                            hash_list = [leaf[0] for leaf in clade.leaf_list]
+                            # Nmae of the collapsed clade should include all taxon names of hash
+                            for _hash in hash_list:
+                                if _hash in hash_dict:
+                                    bygene_taxon_dict[taxon].add(
+                                        hash_dict[_hash][1].strip()
+                                    )
+
+                        # Make bygene_taxon_dict to string
+                        for taxon in bygene_taxon_dict:
+                            sp_list = []
+                            for _taxon in bygene_taxon_dict[taxon]:
+                                if "sp." in _taxon:
+                                    for _t in _taxon.split("/"):
+                                        sp_list.append(_t)
+
+                            sp_list = list(set(sp_list))
+
+                            nonsp_list = []
+                            for _taxon in bygene_taxon_dict[taxon]:
+                                if not ("sp." in _taxon):
+                                    for _t in _taxon.split("/"):
+                                        nonsp_list.append(_t)
+                            nonsp_list = list(set(nonsp_list))
+
+                            sp_string = "/".join(
+                                [
+                                    "/".join(sorted(nonsp_list)),
+                                    "/".join(sorted(sp_list)),
                                 ]
+                            )
 
-                                available_set = set()
-                                for _hash in hash_list:
-                                    if _hash in hash_dict:
-                                        available_set.add(hash_dict[_hash][1])
+                            if sp_string.startswith("/"):
+                                sp_string = sp_string[1:]
+                            if sp_string.endswith("/"):
+                                sp_string = sp_string[:-1]
 
-                                species_text = "/".join(sorted(list(available_set)))
+                            bygene_taxon_dict[taxon] = sp_string
 
-                                if not (".sp") in taxon[1]:
-                                    species_text = f"{taxon[1]}/{species_text}"
-
-                                bygene_taxon_dict[taxon] = species_text
-
-                        # Change taxon name
-                        tmp_taxon_list = [key for key in tree_info.collapse_dict]
+                        ## Update tree_info.collapse_dict with designated name
+                        # Perform in two steps in order to take collapse_dict safe
+                        tmp_taxon_list = list(set(tree_info.collapse_dict.keys()))
+                        dumped_dict = {}
                         for taxon in tmp_taxon_list:
-                            if "sp." in taxon[1]:
+                            if not (
+                                f"tmp {bygene_taxon_dict[taxon]}"
+                                in tree_info.collapse_dict
+                            ):
                                 tree_info.collapse_dict[
-                                    (taxon[0], bygene_taxon_dict[taxon])
+                                    (taxon[0], f"tmp {bygene_taxon_dict[taxon]}")
                                 ] = tree_info.collapse_dict.pop(taxon)
+                                dumped_dict[taxon] = (
+                                    taxon[0],
+                                    f"tmp {bygene_taxon_dict[taxon]}",
+                                )
+                            else:
+                                cnt = 1
+                                while 1:
+                                    if (
+                                        not (f"tmp {bygene_taxon_dict[taxon]} {cnt}")
+                                        in tree_info.collapse_dict
+                                    ):
+                                        tree_info.collapse_dict[
+                                            (
+                                                taxon[0],
+                                                f"tmp {bygene_taxon_dict[taxon]} {cnt}",
+                                            )
+                                        ] = tree_info.collapse_dict.pop(taxon)
+                                        dumped_dict[taxon] = (
+                                            taxon[0],
+                                            f"tmp {bygene_taxon_dict[taxon]} {cnt}",
+                                        )
+                                        break
+                                    else:
+                                        cnt += 1
+
+                        for taxon in tmp_taxon_list:
+                            try:
+                                tree_info.collapse_dict[
+                                    (taxon[0], f"{bygene_taxon_dict[taxon]}")
+                                ] = tree_info.collapse_dict.pop(dumped_dict[taxon])
+
+                                for collapse_info in tree_info.collapse_dict[
+                                    (taxon[0], f"{bygene_taxon_dict[taxon]}")
+                                ]:
+                                    collapse_info.taxon = (
+                                        taxon[0],
+                                        f"{bygene_taxon_dict[taxon]}",
+                                    )
+                            except:
+                                pass
+
                 cnt_sp_adder += len(taxon_set)
 
     # Return sp number fixed tree_info_list
@@ -231,21 +316,24 @@ def synchronize(V, tree_info_list):
 ### Visualization after synchronization
 def pipe_module_tree_visualization(
     tree_info,
-    group,
-    gene,
     V,
     path,
     opt,
 ):
+    patch()
 
+    group = tree_info.group
+    gene = tree_info.gene
     genus_list = V.tup_genus
 
     # Collapse tree branches for visualization
-    tree_info.collapse_tree()
+    taxon_string_list = tree_info.collapse_tree()
 
     # Polish tree image
     tree_info.polish_image(
-        f"{path.out_tree}/{opt.runname}_{group}_{gene}.svg", genus_list
+        f"{path.out_tree}/{opt.runname}_{group}_{gene}.svg",
+        taxon_string_list,
+        genus_list,
     )
 
     # sort taxon order
@@ -277,12 +365,13 @@ def pipe_module_tree_visualization(
                 report.update_species_original(
                     get_genus_species(leaf[2], genus_list=genus_list)
                 )
-                report.update_species_assigned(taxon)
+                report.update_species_assigned(" ".join(taxon))
                 report.ambiguous = collapse_info.clade_cnt
                 report.flat = collapse_info.flat
 
                 report_list.append(report)
 
+        # If more than one taxon exists,
         else:
             for n, collapse_info in enumerate(tree_info.collapse_dict[taxon]):
                 for leaf in collapse_info.leaf_list:
@@ -294,7 +383,7 @@ def pipe_module_tree_visualization(
                     report.update_species_original(
                         get_genus_species(leaf[2], genus_list=genus_list)
                     )
-                    report.update_species_assigned((" ".join(taxon), f"{n+1}"))
+                    report.update_species_assigned((f"{taxon[0]} {taxon[1]} {n+1}"))
                     report.ambiguous = collapse_info.clade_cnt
                     report.flat = collapse_info.flat
 
@@ -305,8 +394,9 @@ def pipe_module_tree_visualization(
 
 ### For all datasets, multiprocessing part
 def pipe_tree_interpretation(V, path, opt):
-    tree_interpretation_opt = []
 
+    # Generate tree_interpretation opt to run
+    tree_interpretation_opt = []
     for group in V.dict_dataset:
         for gene in V.dict_dataset[group]:
             # draw tree only when query sequence exists
@@ -327,7 +417,7 @@ def pipe_tree_interpretation(V, path, opt):
                     )
                 )
 
-    # Tree interpretation - outgroup, reconstruction(solve_flat), collapsing
+    ## Tree interpretation - outgroup, reconstruction(solve_flat), collapsing
     if opt.verbose < 3:
         p = mp.Pool(opt.thread)
         tree_info_list = p.starmap(
@@ -343,32 +433,41 @@ def pipe_tree_interpretation(V, path, opt):
             for option in tree_interpretation_opt
         ]
 
-    # Synchronize sp. numbers
-    tree_info_list = synchronize(V, tree_info_list)
-
-    tree_interpretation_result = []
+    tree_info_list = synchronize(V, path, tree_info_list)
+    # Generate visualization option to run
+    tree_visualization_opt = []
     for tree_info in tree_info_list:
-        tree_interpretation_result.append(
-            pipe_module_tree_visualization(
-                tree_info, tree_info.group, tree_info.gene, V, path, opt
-            )
+        tree_visualization_opt.append((tree_info, V, path, opt))
+
+    ## Tree visualization
+    if opt.verbose < 3:
+        p = mp.Pool(opt.thread)
+        tree_visualization_result = p.starmap(
+            pipe_module_tree_visualization, tree_visualization_opt
         )
+        p.close()
+        p.join()
+
+    else:
+        # non-multithreading mode for debugging
+        tree_visualization_result = [
+            pipe_module_tree_visualization(*option) for option in tree_visualization_opt
+        ]
 
     # Collect identifiation result to V
-    for raw_result in tree_interpretation_result:
-        if raw_result is not (None):
-            for result in raw_result:
-                FI = V.dict_hash_FI[result.hash]
-                if result.gene == "concatenated" or opt.concatenate is False:
-                    # logging.debug("Updating final species")
-                    FI.final_species = result.species_assigned
-                    FI.species_identifier = result.ambiguous
-                    if result.flat is True:
-                        FI.flat.append("concatenated")
-                else:
-                    # logging.debug("Not updating final species")
-                    FI.bygene_species[result.gene] = result.species_assigned
-                    if result.flat is True:
-                        FI.flat.append(result.gene)
+    for report_list in tree_visualization_result:
+        for report in report_list:
+            FI = V.dict_hash_FI[report.hash]
+            # Concatenated
+            if report.gene == "concatenated":
+                FI.final_species = report.species_assigned
+                FI.species_identifier = report.ambiguous
+                if report.flat is True:
+                    FI.flat.append("concatenated")
+            # Non concatenated
+            else:
+                FI.bygene_species[report.gene] = report.species_assigned
+                if report.flat is True:
+                    FI.flat.append(report.gene)
 
     return V, path, opt
